using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Security;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect.BaseClasses;

public abstract class FunctionalBaseClass {
   // protected readonly FitConnectApiService ApiService;

    protected IOAuthService OAuthService;
    protected ISubmissionService SubmissionService;
    protected IRouteService RouteService;

    public readonly IEncryption Encryption;
    protected readonly ILogger? Logger;
    protected readonly X509Certificate2 Certificate;
    internal Client Owner { get; set; }


    /// <summary>
    ///     Constructor for the FunctionalBaseClass
    /// </summary>
    /// <param name="endpoints">FitConnect endpoints</param>
    /// <param name="certificate">The Encryption certificate</param>
    /// <param name="logger">ILogger implementation</param>
    /// <example>
    ///     new Sender(logger, FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development))
    /// </example>
    protected FunctionalBaseClass(FitConnectEndpoints? endpoints,
        X509Certificate2 certificate,
        ILogger? logger,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService) {
        Endpoints = endpoints ??
                    FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development);

        Logger = logger;
        OAuthService = oAuthService;
        SubmissionService = submissionService;
        RouteService = routeService;
        Certificate = certificate;
        Encryption = new RsaEncryption(logger, certificate);
    }

    public FitConnectEndpoints Endpoints { get; }


    /// <summary>
    /// Requesting the OAuth-Token
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    public Task<OAuthAccessToken?> AuthenticateAsync(string clientId, string clientSecret,
        string? scope) {
        return OAuthService.AuthenticateAsync(clientId, clientSecret, scope);
    }

    /// <summary>
    ///     Receive the SET data
    /// </summary>
    /// <returns></returns>
    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }
}
