using System;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;
using Autofac;
using FitConnect;
using FitConnect.Models;
using FitConnect.Security;
using FitConnect.Services;
using Moq;
using NUnit.Framework;

namespace FluentApiTest;

public class FluentSenderTests {
    private Client _client = null!;
    private string clientId = "development";
    private string clientSecret = "development";
    private IContainer _container;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        var builder = new ContainerBuilder();

        var routeService = new Mock<IRouteService>();
        routeService.Setup(r => r.GetDestinationIdAsync(It.IsAny<string>(), It.IsAny<string>(),
                It.IsAny<string>(), It.IsAny<string?>()))
            .Returns(() => Task.Run(() => "destination"));

        builder.Register(c => new Mock<IOAuthService>().Object).As<IOAuthService>();
        builder.Register(c => new Mock<ISubmissionService>().Object).As<ISubmissionService>();
        builder.Register(c => routeService.Object).As<IRouteService>();
        _container = builder.Build();
    }

    [SetUp]
    public void Setup() {
        var certificate = RsaEncryption.CreateSelfSignedCertificate();

        _client = new Client(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            "", "", certificate, certificate, container: _container);
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServices() {
        _client.Sender
            .Authenticate(clientId, clientSecret)
            .CreateSubmission(new Submission {
                Attachments = new List<Attachment>(), ServiceType = new ServiceType(),
                Destination = new Destination { LeikaKey = "09929292772", AreaId = "948488" }
            })
            .UploadAttachments()
            .SendSubmission(new Metadata(), new Data());
    }
}
